#define LOG_TAG "modbus_rtu"
#include "elog.h"
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <modbus.h>

#include "common.h" 
#include <pthread.h>
#include "modbus_rtu_slave.h"

#define REG_ADDR (header_length+1)
#define REG_NUM (header_length+3)
#define REG_VAL (header_length+3)
#define device_id 2

enum {
  TCP,
  TCP_PI,
  RTU
};

static pthread_mutex_t  modbus_rtu_slave_mutex;
static modbus_mapping_t *mb_mapping = NULL;
//extern uint16 sjzd_modbus_data;
static uint32_t coil_arr[4];//128个线圈

static void modbus_rtu_slave_lock(void)
{
  pthread_mutex_lock(&modbus_rtu_slave_mutex);	
}

static void modbus_rtu_slave_unlock(void)
{
  pthread_mutex_unlock(&modbus_rtu_slave_mutex);	
}

void modbus_rtu_data_fresh(void *data,int is_new)
{
  static int data_index=0;
  modbus_rtu_slave_lock();
 
  modbus_rtu_slave_unlock();
}

//设置modbus线圈值
void modbus_rtu_set_coil()
{
  modbus_rtu_slave_lock();

  modbus_rtu_slave_unlock();
}


//从modbus获取线圈值
void modbus_rtu_get_coil()
{
  modbus_rtu_slave_lock();

  modbus_rtu_slave_unlock();
}

void modbus_rtu_slave_routine(void *arg)
{ 
  int s = -1;
  int i = 0;
  int rc = 0;
  modbus_t *ctx = NULL;
  uint8_t query[MODBUS_TCP_MAX_ADU_LENGTH];//接收缓冲区
  int header_length;
  uint16_t *ptr;
  char temp[18]={10,0,8,1,2,3,0,9,11,12,0,14,0,17,19,20,0,18};
  //int use_backend=RTU;
  int use_backend=TCP;
  uint16_t reg_addr,reg_val,reg_num;

  if(use_backend == TCP) 
  {
    // switch(device_id)
    // {
    //   case 1:
    //         ctx = modbus_new_tcp("192.168.1.51",1502);  
    //         log_d("libmodbus server 192.168.1.51");
    //   break;
    //   case 2:
    //         ctx = modbus_new_tcp("192.168.1.52",1502);  
    //         log_d("libmodbus server 192.168.1.52");
    //   break;
    //   case 3:
    //         ctx = modbus_new_tcp("192.168.1.53",1502);  
    //         log_d("libmodbus server 192.168.1.53");
    //   break;
    //   case 4:
    //         ctx = modbus_new_tcp("192.168.1.54",1502);  
    //         log_d("libmodbus server 192.168.1.54");
    //   break;
    //   default:
    //   break;

    // }
      ctx = modbus_new_tcp("0.0.0.0",1502);

  }  
  else if (use_backend == TCP_PI) 
  {
    ctx = modbus_new_tcp_pi("::0","1502");     
  } 
  else
  {
    //打开端口: 端口，波特率，校验位，数据位，停止位
    ctx = modbus_new_rtu(MODBUS_RTU_SERIAL,MODBUS_RTU_BAUD,'N',8,1);
  }
  
  //设置从机地址
  modbus_set_slave(ctx,SERVER_ID);

  header_length = modbus_get_header_length(ctx);

  modbus_set_debug(ctx,TRUE);
       
 /*mb_mapping = modbus_mapping_new_start_address(UT_BITS_ADDRESS,UT_BITS_NB,UT_INPUT_BITS_ADDRESS,UT_INPUT_BITS_NB,UT_REGISTERS_ADDRESS,UT_REGISTERS_NB_MAX,UT_INPUT_REGISTERS_ADDRESS,UT_INPUT_REGISTERS_NB);*/
  //油色谱总共24个寄存器
  mb_mapping = modbus_mapping_new(MODBUS_MAX_READ_BITS,0,MODBUS_MAX_READ_REGISTERS,0);
  if(mb_mapping == NULL) {
    log_e("Failed to allocate the mapping: %s\n",modbus_strerror(errno));
    modbus_free(ctx);
    return ;
  }
  //初始化寄存器
  modbus_rtu_slave_lock();
  //mb_mapping->tab_registers[0] = 0x000f;

  
  for(i=0;i<18;i++)
    sjzd_modbus_data[221+i]=temp[i];



  modbus_rtu_slave_unlock();

  // Initialize input values that's can be only done server side.
  //modbus_set_bits_from_bytes(mb_mapping->tab_input_bits,0,UT_INPUT_BITS_NB,UT_INPUT_BITS_TAB);

  // Initialize values of INPUT REGISTERS 
  //for(i=0; i < UT_INPUT_REGISTERS_NB; i++) {
  //  mb_mapping->tab_input_registers[i] = UT_INPUT_REGISTERS_TAB[i];
  //}
 while(1)
 {
    if(use_backend == TCP) 
    {
      //log_d("listening........");
      s = modbus_tcp_listen(ctx,1);
      modbus_tcp_accept(ctx,&s);
      
    } 
    else if(use_backend == TCP_PI) 
    {
      s = modbus_tcp_pi_listen(ctx,1);
      modbus_tcp_pi_accept(ctx,&s);
    } 
    else 
    {
      //建立连接
      rc = modbus_connect(ctx);
      if (rc == -1) 
      {
        log_e("unable to connect %s\n",modbus_strerror(errno));
        modbus_free(ctx);
        return ;
      }
      log_d("libmodbus connected");
    }
    log_d("libmodbus inited successful");
    log_d("###########################################################################");
    while (1)
    {
      do{
          memset(query,0,sizeof(query));
          rc = modbus_receive(ctx,query);
        } while (rc == 0);
      if(rc ==-1&&errno!=EMBBADCRC) 
      {
        log_e("parse modbus frame error: %s\n",modbus_strerror(errno));
        break;
      //continue;
      }
      switch(query[header_length])
        {
          case MODBUS_FC_READ_COILS://Read Coils in mb_mapping->tab_bits：IO输出
            //读继电器
              /*coil_arr[0]=relay_get_all(NULL);
              memcpy(mb_mapping->tab_bits,coil_arr,4);*/
            break;
          case MODBUS_FC_READ_DISCRETE_INPUTS://Read Discrete Inputs in mb_mapping->tab_input_bits：IO输入
              log_e("unsurported function 2\n");
              //否定应答
              modbus_reply_exception(ctx,query,MODBUS_EXCEPTION_ILLEGAL_FUNCTION);
              continue;
            break;
          case MODBUS_FC_READ_HOLDING_REGISTERS://Read Holding Registers in mb_mapping->tab_registers:REG输出
              //读寄存器
              /*if((MODBUS_GET_INT16_FROM_INT8(query,REG_ADDR)+MODBUS_GET_INT16_FROM_INT8(query,REG_NUM))>(sizeof(YSP_MODBUS_REGS)/2))
              {
                log_e("reply to this special register address by an exception\n");
                //否定应答
                modbus_reply_exception(ctx,query,MODBUS_EXCEPTION_ILLEGAL_DATA_ADDRESS);
                continue;
              }*/
              //更新寄存器
              modbus_rtu_slave_lock();

              for(i=0;i<(sizeof(sjzd_modbus_data)/2);i++)
              {
                mb_mapping->tab_registers[i] = sjzd_modbus_data[i];
              }
              //memset(&mb_mapping->tab_registers[0],sjzd_modbus_data,sizeof(sjzd_modbus_data));
              modbus_rtu_slave_unlock();
            break;
          case MODBUS_FC_READ_INPUT_REGISTERS://Read Input Registers in mb_mapping->tab_input_registers:REG输入
            //可用于读取测量值等不可上位机修改的量
            break;
          case MODBUS_FC_WRITE_SINGLE_COIL://Write Single Coil:slave_addr+cmd+reg_addr+[FF 00 or 00 00]+crc corresponding to mb_mapping->tab_bits
            //写单个继电器
            reg_addr=MODBUS_GET_INT16_FROM_INT8(query,REG_ADDR);
            reg_val=MODBUS_GET_INT16_FROM_INT8(query,REG_VAL);
            /*if(reg_addr<32)
            {
              relay_set(reg_addr,(reg_val==0xFF00?1:0),1);
            }
            else
            {
              switch(reg_addr)
              {
                case 32:
                  break;
                case 33:
                  break;
                default:
                  break;
              }
            }*/     
            break;
          case MODBUS_FC_WRITE_SINGLE_REGISTER://Write Single Register:slave_addr+cmd+reg_addr+reg_val+crc corresponding to mb_mapping->tab_registers
            //写单个寄存器
            reg_addr=MODBUS_GET_INT16_FROM_INT8(query,REG_ADDR);
            reg_val=MODBUS_GET_INT16_FROM_INT8(query,REG_VAL);
            break;
          case MODBUS_FC_WRITE_MULTIPLE_COILS://Write Multiple Coils:slave_addr+cmd+reg_addr+coil_num+bit_arr+crc corresponding to mb_mapping->tab_bits
            //写多个继电器
            break;
          case MODBUS_FC_WRITE_MULTIPLE_REGISTERS://Write Multiple Registers:slave_addr+cmd+reg_addr+reg_num+reg_val_arr+crc corresponding to mb_mapping->tab_registers
            //写多个寄存器
            reg_addr=MODBUS_GET_INT16_FROM_INT8(query,REG_ADDR);
            reg_num=MODBUS_GET_INT16_FROM_INT8(query,REG_NUM);       
            break;
          default://
            break;
        }
      
      //正常应答
      rc = modbus_reply(ctx,query,rc,mb_mapping);
      if(rc==-1)
      {
        log_e("reply modbus frame error: %s\n",modbus_strerror(errno));
        break;
      }
    }


    if(use_backend==TCP) 
    {
      if(s != -1) 
      {
        close(s);
      }
    }

    log_e("quit the loop: %s\n",modbus_strerror(errno));
    
    
  }
  log_d("********modbus end************");
  // For RTU 
  modbus_mapping_free(mb_mapping);
  modbus_close(ctx);
  modbus_free(ctx);
}

int modbus_rtu_slave_init(void)
{
  pthread_t modbus_rtu_slave_thread;
  if(pthread_mutex_init(&modbus_rtu_slave_mutex,NULL)!=0)
   {
     log_e("init modbus_rtu_slave_mutex failed\r\n");
     return -1;	
   }  
  if((modbus_rtu_slave_thread=task_create(modbus_rtu_slave_routine,NULL,0,0))==-1)
   {
     log_e("create modbus_rtu_slave_routine failed\n");
     return -2;
   }
  return 0;
}
