#define LOG_TAG "sjzd_serial"
#include "sjzd_serial.h"
#include "elog.h"
#include "crc.h"

typedef struct
{
  /* data */
  char *virtual_serial;
  char *system_serial;
}SERIAL_MAP_TYPE;


SERIAL_MAP_TYPE serial_map_tbl[]={
{"RS485_1","/dev/ttymxc1"},
{"RS485_2","/dev/ttymxc2"},
{"RS485_3","/dev/ttymxc3"},
};
static int sjzd_serial_fd = -1;//串口句柄

static char *sjzd_get_port_by_name(char *vname)
{
  uint32_t i;
  for(i=0;i<(sizeof(serial_map_tbl)/sizeof(SERIAL_MAP_TYPE));i++)
  {
      //printf("tbl[%d]:%s,%s\n",i,serial_map_tbl[i].virtual_serial,vname);
      if(strcmp(serial_map_tbl[i].virtual_serial,vname)==0)
      {
          return serial_map_tbl[i].system_serial;
      }
  }
  return NULL;
}

int32_t sjzd_serial_init(char *port,int baud,char databit,char parity)
{
  char *dev_name;
  if(port==NULL)
  {
    log_e("port is null in sjzd_serial_init");
    return -1;
  }
  if((dev_name=sjzd_get_port_by_name(port))==NULL) 
  {
    log_e("get %s failed",port);
    return -2;
  }
  if((sjzd_serial_fd=SerialOpen(dev_name,baud,databit,1,parity))>=0)//0:无校验 1:奇校验 2:偶校验
  {
    return 0;
  }
  log_e("open %s failed",dev_name);
  return -3;
}

int32_t sjzd_serial_close(void)
{
  if(sjzd_serial_fd!=-1)
  {
    close(sjzd_serial_fd);
    sjzd_serial_fd=-1;   
  }
  return 0;
}

int sjzd_recv_data(SJZD_SERIAL_DEF *pPortParam, u_char *buf, int len, uint16_t usNBytes)
{
	int  				i, flag = -1;
  uint8_t crc_val[2];
  static int fault=0;

	for (i = 0; i < len; i++)
	{
		switch (pPortParam->m_iRevStatus)
		{
			case 0:			// 0xAA
				pPortParam->m_iRecvLen = 0;
				pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen++] = buf[i];
				if (0xAA == buf[i])
					pPortParam->m_iRevStatus++;
				else
        {
					pPortParam->m_iRevStatus = 18;
          log_e("1:[%d]%02X != AA", i+1, buf[i]);
        }
				break;
			case 1:			// addr
        if((0<buf[i]) && (3 >= buf[i]))
				{
          pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen++] = buf[i];
	  			pPortParam->m_iRevStatus++;
        }
        else
        {
          log_e("2:[%d]addr=%d-----", i+1, buf[i]);
          pPortParam->m_iRevStatus = 18;
        }
				break;
			case 2:			// len低字节
				pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen++] = buf[i];
				pPortParam->m_iRevStatus++;
				break;
			case 3:			// len高字节
				pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen++] = buf[i];
				pPortParam->m_iRevStatus++;
				pPortParam->m_iNeedRevLength = pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen - 2] + buf[i] * 256 + 3;
        if(pPortParam->m_iNeedRevLength > usNBytes+10)
        {
          log_e("3:[%d]m_iNeedRevLength=%d > %d(usNBytes%d+8) %02X%02X -----", i+1,pPortParam->m_iNeedRevLength, usNBytes+8, usNBytes, pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen-2], buf[i]);
          pPortParam->m_iRevStatus = 18;
        }
				break;
			case 4:			// cmd
				pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen++] = buf[i];
				pPortParam->m_iNeedRevLength--;
        if(4 == buf[i])
        {
          if(fault == 5)
          {
            log_e("test error,fault=%d", fault);
            fault = 0;
            return 0;
          }
          else
            fault++;
        }
				if ((0 < buf[i]) && (6 > buf[i]))
					pPortParam->m_iRevStatus++;
				else
				{
          log_e("4:buf[%d]cmd=%d-----", i+1, buf[i]);
          pPortParam->m_iRevStatus = 18;
        }	
				break;
			case 5:	// 正确接收数据
				pPortParam->m_iNeedRevLength--;
				pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen++] = buf[i];
				if (pPortParam->m_iNeedRevLength > 0)
					break;
				if (buf[i] != 0x55)
				{
          log_e("5:[%d]%02X != 55", i+1, buf[i]);
					pPortParam->m_iRevStatus = 18;
					break;
				}

        crc16_ex(pPortParam->m_au8RecvBuf,pPortParam->m_iRecvLen-3,crc_val);//

        if((crc_val[0]==(pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen - 2]))&&(crc_val[1]==pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen - 3]))
          flag = 1;
        if((crc_val[0]==(pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen - 3]))&&(crc_val[1]==pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen - 2]))
          flag = 1;
				if (1 == flag)
				{
          memmove(buf, pPortParam->m_au8RecvBuf, pPortParam->m_iRecvLen);
          return pPortParam->m_iRecvLen;
				}
				else
				{
          log_e("6:%02X%02X %02X%02X -----", crc_val[0], crc_val[1], pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen - 2], pPortParam->m_au8RecvBuf[pPortParam->m_iRecvLen - 3]);
					pPortParam->m_iRevStatus = 18;
          pPortParam->m_iRecvLen = 0;
				}
				break;
			case 255:// 错误接收数据
			default:
				if (buf[i] == 0xAA)
				{
					pPortParam->m_iRevStatus = 1;
					pPortParam->m_iRecvLen = 1;
					pPortParam->m_au8RecvBuf[0] = buf[i];
				}
				else if (buf[i] == 0x55)
				{
					pPortParam->m_iRevStatus = 0;
					pPortParam->m_iRecvLen = 0;
				}
				else
				{
						pPortParam->m_iRecvLen = 0;
				}
				break;
		}
	}
  return 0;
}

int32_t sjzd_serial_read(uint8_t *pucBuffer,uint16_t usNBytes,uint16_t *usNBytesRead)
{
  int rx_len;
  SJZD_SERIAL_DEF *sjzdport;

  rx_len=SerialReadEx(sjzd_serial_fd,pucBuffer,usNBytes,MODBUS_TIMEOUT);
  sjzdport=(SJZD_SERIAL_DEF *)malloc(sizeof(SJZD_SERIAL_DEF));
  if(sjzdport==NULL)
  {
    log_e("sjzdport alloc file buf failed");
    return 0;
  }
  memset(sjzdport, 0, sizeof(SJZD_SERIAL_DEF));
    log_e("rx_len=%d", rx_len);
  rx_len = sjzd_recv_data(sjzdport, pucBuffer, rx_len, usNBytes);
  if(usNBytesRead!=NULL)
  {
    if(rx_len>0)
    {
      *usNBytesRead=rx_len;
    }
    else
    {
      /* code */
      *usNBytesRead=0;
    }   
  }
  if(NULL != sjzdport)
  {
    free((void*)sjzdport);
    sjzdport = NULL;
  }
  return rx_len;
}

int32_t sjzd_serial_write(uint8_t *pucBuffer,uint16_t usNBytes)
{
  return SerialWrite(sjzd_serial_fd,pucBuffer,usNBytes);
}